import json
import grpc
from zeebe.base import ZeebeBase
from zeebe_grpc import gateway_pb2, gateway_pb2_grpc
from zeebe.exceptions import ZeebeClientException


class ZeebeClient(ZeebeBase):
    """Represents a zeebe Client"""

    def __init__(self, zeebe_address: str = None, client_id: str = None, client_secret: str = None):
        super().__init__(zeebe_address, client_id, client_secret)

    def topology(self):
        return self.stub.Topology(gateway_pb2.TopologyRequest())

    def deploy_workflow(self, *args):
        workflows = [
            gateway_pb2.WorkflowRequestObject(
                type=gateway_pb2.WorkflowRequestObject.BPMN,
                definition=open(workflow, "rb").read(),
            ) for workflow in args
        ]

        return self.stub.DeployWorkflow(gateway_pb2.DeployWorkflowRequest(workflows=workflows))

    def create_workflow_instance(self, process_id, variables, version=-1):
        return self.stub.CreateWorkflowInstance(
             gateway_pb2.CreateWorkflowInstanceRequest(
                 bpmnProcessId=process_id,
                 version=version,
                 variables=json.dumps(variables)
             )
        )

    def create_workflow_instance_with_result(self, process_id, variables, version=-1):
        return self.stub.CreateWorkflowInstanceWithResult(
            gateway_pb2.CreateWorkflowInstanceRequest(
                bpmnProcessId=process_id,
                version=version,
                variables=json.dumps(variables)
            )
        )
